<?php
namespace Zeedhi\Framework\DB\StoredProcedure\Strategies;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\DataBaseStrategy;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class SQLServerStrategy implements DataBaseStrategy{

    /** @var Connection */
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    /**
     * @param StoredProcedure $procedure   The procedure to be executed.
     * @param array           $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $outputValues = array();
        $params = array();

        foreach ($procedure->getParams() as $key => $param) {
            $outputValues[$param->getName()] = $param->getValue();
            $params[$key+1] = array(
                &$outputValues[$param->getName()],
                $param->isOutput() ? SQLSRV_PARAM_OUT : SQLSRV_PARAM_IN
            );
        }

        $this->connection->prepare($queryStmt)->execute($params);
        return $outputValues;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return '@' . $param->getName() . ' = ? ';
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        $queryStmt = "EXECUTE " . $procedure->getName() . " " . $paramString;
        return $queryStmt;
    }
}